import "./cas.config";
import { CasModel } from "./cas.model";

const { shortcut, store } = BI.Decorators;


function createTipIcon(text) {
    return {
        type: "bi.bubble_combo",
        direction: "bottom,right",
        el: {
            type: "bi.icon_label",
            cls: "detail-font"
        },
        popup: {
            el: {
                type: "bi.vertical",
                items: [
                    {
                        el: {
                            type: "bi.label",
                            textAlign: "left",
                            whiteSpace: "normal",
                            text,
                        },
                        vgap: 10,
                        hgap: 10,
                    }
                ]
            }
        }
    };
}

function createTextEditorLabel(options) {

    const { ref, invisible, text, value, watermark, tip } = options;
    return {
        type: "bi.vertical_adapt",
        invisible,
        columnSize: [132, ""],
        items: [
            {
                el: {
                    type: "bi.label",
                    textAlign: "left",
                    text,
                    cls: "dec-font-weight-bold"
                }
            }, {
                el: {
                    type: "bi.text_editor",
                    ref,
                    allowBlank: true,
                    errorText: BI.i18nText("Dec-Error_Null"),
                    watermark,
                    value,
                    width: 300,
                }
            }, {
                el: createTipIcon(tip),
                hgap: 10
            }
        ]
    };
}

@shortcut()
@store(CasModel)
export class Cas extends BI.Widget {
    static xtype = "dec.plugin.system.cas";

    props = {
        baseCls: "dec-plugin-system-cas bi-card"
    };

    beforeInit(render) {
        this.store.initCasConfig(render);
    }

    render() {

        let { turnOn, logoutTurnOn, casLogoutUrl } = this.model.casConfig;

        const { casServerUrl, casLoginUrl } = this.model.casConfig.filterStrategy;
        let casConfigContent;

        return {
            type: "bi.vertical",
            hgap: 10,
            items: [
                {
                    el: {
                        type: "bi.left_right_vertical_adapt",
                        cls: "bi-border-bottom",
                        height: 40,
                        items: {
                            left: [
                                {
                                    type: "bi.label",
                                    text: BI.i18nText("Dec-Basic_Config")
                                }
                            ],
                            right: [
                                {
                                    type: "bi.button",
                                    text: BI.i18nText("Dec-Basic_Save"),
                                    handler: () => {
                                        this.validation() && this.store.saveCasConfig(this.getValue());
                                    }
                                }
                            ]
                        }
                    }
                }, {
                    el: {
                        type: "bi.vertical",
                        items: [
                            {
                                el: {
                                    type: "bi.vertical_adapt",
                                    columnSize: [132, ""],
                                    items: [
                                        {
                                            el: {
                                                type: "bi.label",
                                                textAlign: "left",
                                                text: BI.i18nText("Dec-Plugin_System_Cas_Turn_On"),
                                                cls: "dec-font-weight-bold"
                                            }
                                        }, {
                                            el: {
                                                type: "bi.switch",
                                                ref: (_ref) => {
                                                    this.casTurnOnButton = _ref;
                                                },
                                                selected: turnOn,
                                                handler: () => {
                                                    turnOn = !turnOn;
                                                    casConfigContent.setVisible(turnOn);
                                                }
                                            }
                                        }
                                    ]
                                },
                                tgap: 15,
                            }, {
                                el: {
                                    type: "bi.vertical",
                                    ref: (_ref) => {
                                        casConfigContent = _ref;
                                    },
                                    invisible: !turnOn,
                                    items: [
                                        {
                                            el: {
                                                type: "bi.vertical_adapt",
                                                items: [
                                                    {
                                                        el: createTextEditorLabel({
                                                            text: BI.i18nText("Dec-Plugin_System_Cas_Server_URL"),
                                                            ref: (_ref) => {
                                                                this.casServerUrl = _ref;
                                                            },
                                                            value: casServerUrl,
                                                            watermark: "http://www.example.com/cas",
                                                            tip: BI.i18nText("Dec-Plugin_System_Cas_Server_URL_Tip")
                                                        })
                                                    }, {
                                                        el: {
                                                            type: "bi.text_button",
                                                            cls: "bi-high-light",
                                                            text: BI.i18nText("Dec-Test_Connection"),
                                                            handler: () => {
                                                                const url = this.casServerUrl.getValue();
                                                                if (!BI.isKey(url)) {
                                                                    this.casServerUrl.editor._setErrorVisible(true);
                                                                    return;
                                                                }
                                                                this.casServerUrl.editor._setErrorVisible(false);
                                                                this.testConnection();
                                                            }
                                                        }
                                                    }
                                                ]
                                            }
                                        }, {
                                            el: createTextEditorLabel({
                                                text: BI.i18nText("Dec-Plugin_System_Cas_Login_URL"),
                                                ref: (_ref) => {
                                                    this.casLoginUrl = _ref;
                                                },
                                                value: casLoginUrl,
                                                watermark: "http://www.example.com/cas/login",
                                                tip: BI.i18nText("Dec-Plugin_System_Cas_Login_URL_Tip")
                                            })
                                        }, {
                                            el: {
                                                type: "bi.vertical_adapt",
                                                columnSize: [132, ""],
                                                items: [
                                                    {
                                                        el: {
                                                            type: "bi.label",
                                                            textAlign: "left",
                                                            text: BI.i18nText("Dec-Plugin_System_Cas_Logout_Turn_On"),
                                                            cls: "dec-font-weight-bold"
                                                        }
                                                    }, {
                                                        el: {
                                                            type: "bi.switch",
                                                            ref: (_ref) => {
                                                                this.casLogoutButton = _ref;
                                                            },
                                                            selected: logoutTurnOn,
                                                            handler: () => {
                                                                logoutTurnOn = !logoutTurnOn;
                                                                this.casLogoutUrlContent.setVisible(logoutTurnOn);
                                                            }
                                                        }
                                                    }, {
                                                        el: createTipIcon(BI.i18nText("Dec-Plugin_System_Cas_Logout_Tip")),
                                                        hgap: 10
                                                    }
                                                ]
                                            }
                                        }, {
                                            el: BI.extend({
                                                ref: (_ref) => {
                                                    this.casLogoutUrlContent = _ref;
                                                },
                                            }, createTextEditorLabel({
                                                text: BI.i18nText("Dec-Plugin_System_Cas_Logout_Url"),
                                                ref: (_ref) => {
                                                    this.casLogoutUrl = _ref;
                                                },
                                                invisible: !logoutTurnOn,
                                                value: casLogoutUrl,
                                                watermark: "http://www.example.com/cas/logout",
                                                tip: BI.i18nText("Dec-Plugin_System_Cas_Logout_Url_Tip"),
                                            }))
                                        }, {
                                            el: this.createExpander()
                                        }
                                    ],
                                    vgap: 15,
                                },
                            }
                        ]
                    }
                },
            ]
        };
    }

    createExpander() {
        let expanderIcon = null;
        let expandPopup = null;
        let expanded = false;

        const { ignoreH5, ignoreLoginPage } = this.model.casConfig;
        return {
            type: "bi.vertical",
            items: [
                {
                    el: {
                        type: "bi.vertical_adapt",
                        items: [
                            {
                                el: {
                                    type: "bi.icon_change_button",
                                    ref: (_ref) => {
                                        expanderIcon = _ref;
                                    },
                                    iconCls: "triangle-collapse-font",
                                    handler: () => {
                                        expanded = !expanded;
                                        expanderIcon.setIcon(expanded ? "triangle-expand-font" : "triangle-collapse-font");
                                        expandPopup.setVisible(expanded);
                                    }
                                },
                                rgap: 5,
                            }, {
                                el: {
                                    type: "bi.text_button",
                                    text: BI.i18nText("Dec-Plugin_System_Cas_Ignore_Request"),
                                    handler: () => {
                                        expanded = !expanded;
                                        expanderIcon.setIcon(expanded ? "triangle-expand-font" : "triangle-collapse-font");
                                        expandPopup.setVisible(expanded);
                                    }
                                }
                            }
                        ]
                    },
                }, {
                    el: {
                        type: "bi.vertical",
                        ref: (_ref) => {
                            expandPopup = _ref;
                        },
                        vgap: 15,
                        invisible: true,
                        items: [
                            {
                                el: {
                                    type: "bi.vertical_adapt",
                                    columnSize: [132, ""],
                                    items: [
                                        {
                                            el: {
                                                type: "bi.label",
                                                textAlign: "left",
                                                text: BI.i18nText("Dec-Plugin_System_Cas_H5_Request"),
                                                cls: "dec-font-weight-bold"
                                            }
                                        }, {
                                            el: {
                                                type: "bi.switch",
                                                ref: (_ref) => {
                                                    this.ignoreH5Button = _ref;
                                                },
                                                selected: ignoreH5,
                                                handler: () => {

                                                }
                                            }
                                        }, {
                                            el: createTipIcon(BI.i18nText("Dec-Plugin_System_Cas_H5_Request_Tip")),
                                            hgap: 10
                                        }
                                    ]
                                }
                            }, {
                                el: {
                                    type: "bi.vertical_adapt",
                                    columnSize: [132, ""],
                                    items: [
                                        {
                                            el: {
                                                type: "bi.label",
                                                textAlign: "left",
                                                text: BI.i18nText("Dec-Plugin_System_Cas_Login_Page_Request"),
                                                cls: "dec-font-weight-bold"
                                            }
                                        }, {
                                            el: {
                                                type: "bi.switch",
                                                ref: (_ref) => {
                                                    this.ignoreLoginPageButton = _ref;
                                                },
                                                selected: ignoreLoginPage,
                                                handler: () => {

                                                }
                                            }
                                        }, {
                                            el: createTipIcon(BI.i18nText("Dec-Plugin_System_Cas_Login_Page_Request_Tip")),
                                            hgap: 10
                                        }
                                    ]
                                }
                            },
                        ]
                    }
                }
            ]
        };
    }

    testConnection() {
        const url = this.casServerUrl.getValue();
        this.store.textCasConnection(url).then(() => {
            this.casTestMasker.setStatus({
                status: 2,
            });
        }).catch((error) => {
            this.casTestMasker.setStatus({
                status: 3,
                message: error.errorMsg,
            });
        });
        this.createMasker();
    }

    createMasker() {
        const name = "cas-test";
        const upload = {
            type: "dec.test.status.reload",
            waitingText: BI.i18nText("Dec-Connection_Test_Waiting"),
            successText: BI.i18nText("Dec-Connection_Test_Success"),
            failText: BI.i18nText("Dec-Plugin_System_Cas_Connection_Test_Failed"),
            reloadText: BI.i18nText("Dec-Connection_Reconnect"),
            listeners: [
                {
                    eventName: "EVENT_CLOSE",
                    action: () => {
                        BI.Maskers.remove(name);
                    },
                }, {
                    eventName: "EVENT_RELOAD",
                    action: () => {
                        this.testConnection();
                    },
                }
            ],
            ref: (_ref) => {
                this.casTestMasker = _ref;
            },
        };
        BI.Maskers.create(name, null, {
            render: upload,
        });
        BI.Maskers.show(name);
    }

    validation() {
        const turnOn = this.casTurnOnButton.isSelected();
        let valid = true;
        if (turnOn) {
            const casLoginUrl = this.casLoginUrl.getValue();
            const casServerUrl = this.casServerUrl.getValue();
            if (!BI.isKey(casServerUrl)) {
                this.casServerUrl.editor._setErrorVisible(true);
                valid = false;
            }
            if (!BI.isKey(casLoginUrl)) {
                this.casLoginUrl.editor._setErrorVisible(true);
                valid = false;
            }
            const logoutTurnOn = this.casLogoutButton.isSelected();
            if (logoutTurnOn) {
                const casLogoutUrl = this.casLogoutUrl.getValue();
                if (!BI.isKey(casLogoutUrl)) {
                    this.casLogoutUrl.editor._setErrorVisible(true);
                    valid = false;
                }
            }
        }

        return valid;
    }

    getValue() {

        return {
            casLogoutUrl: this.casLogoutUrl.getValue(),
            filterStrategy: {
                type: "jasig",
                casLoginUrl: this.casLoginUrl.getValue(),
                casServerUrl: this.casServerUrl.getValue()
            },
            https: document.location.protocol === "https:",
            ignoreLoginPage: this.ignoreLoginPageButton.isSelected(),
            ignoreH5: this.ignoreH5Button.isSelected(),
            logoutTurnOn: this.casLogoutButton.isSelected(),
            turnOn: this.casTurnOnButton.isSelected()
        };
    }
}
